import base64

from datetime import datetime, timedelta
from time import sleep

from typing import (
    Dict,
    Optional,
    List,
)
from xml.etree import cElementTree as eTree


from actions.eft import (
    check_eft_is_valid,
    EftConstants,
    get_electronic_tenders,
    has_electronic_payment,
)

from helper import translate_pos_label
from mw_helper import show_message_options_dialog
from old_helper import convert_from_utf_to_localtime

import sysactions

from application.interactor import OrderInteractor
from custom_sysactions import user_authorized_to_execute_action
from manager.reports import justification_report
from utilfunctions import (
    get_custom_property_value,
    is_stored_order,
)
from utilconfigurations import must_require_void_reason
from utils import (
    get_void_reason_id,
    void_order,
    void_fiscal_order,
    get_canceled_by,
    get_fixed_void_reason,
    do_confirm_void,
)
from .do_void_remote_order import (
    void_remote_order,
)
from .. import logger
from ..config.is_tef_available import check_tef_connectivity
from utilfunctions import send_order_canceled_event
from ..eft.cancel_eft_transaction import clear_multiple_tenders

VOID_ATTEMPTS = 3

@sysactions.action
def do_void_order(
    pos_id,                         # type: Optional[str]
    fiscal_void='false',            # type: Optional[str]
    order_id="",                    # type: Optional[str]
    show_cancel_message=True,       # type: Optional[bool]
    is_mobile=False,                # type: Optional[bool]
    eft_processor="",               # type: Optional[str]
    auth_data=None,                 # type: Optional[str]
    from_timeout=False,				# type: Optional[bool]
    received_void_reason=None,      # type: Optional[str]
):
    # type: (...) -> bool

    model = sysactions.get_model(posid=pos_id)
    stored_order = is_stored_order(model=model)

    if from_timeout:
        _void_order(
            pos_id=pos_id,
            fiscal_void=fiscal_void,
            order_id=order_id,
            show_cancel_message=show_cancel_message,
            is_mobile=is_mobile,
            eft_processor=eft_processor,
            auth_data=auth_data,
            received_void_reason=received_void_reason,
            order_is_stored=stored_order,
        )
        return False

    if stored_order:
        return do_void_stored_order(
            pos_id,
            fiscal_void=fiscal_void,
            order_id=order_id,
            show_cancel_message=show_cancel_message,
            is_mobile=is_mobile,
            eft_processor=eft_processor,
            auth_data=auth_data,
            received_void_reason=received_void_reason,
        )
    else:
        return do_void_regular_order(
            pos_id,
            fiscal_void=fiscal_void,
            order_id=order_id,
            show_cancel_message=show_cancel_message,
            is_mobile=is_mobile,
            eft_processor=eft_processor,
            auth_data=auth_data,
            received_void_reason=received_void_reason,
        )


@user_authorized_to_execute_action
def do_void_regular_order(
    pos_id,                 # type: str
    fiscal_void,            # type: str
    order_id,               # type: str
    show_cancel_message,    # type: bool
    is_mobile,              # type: bool
    eft_processor,          # type: str
    auth_data,              # type: str
    received_void_reason,   # type: str
):
    # type: (...) -> bool

    return _void_order(
        pos_id=pos_id,
        fiscal_void=fiscal_void,
        order_id=order_id,
        show_cancel_message=show_cancel_message,
        is_mobile=is_mobile,
        eft_processor=eft_processor,
        auth_data=auth_data,
        received_void_reason=received_void_reason,
    )


@user_authorized_to_execute_action
def do_void_stored_order(
    pos_id,                 # type: str
    fiscal_void,            # type: str
    order_id,               # type: str
    show_cancel_message,    # type: bool
    is_mobile,              # type: bool
    eft_processor,          # type: str
    auth_data,              # type: str
    received_void_reason,   # type: str
):
    # type: (...) -> bool

    return _void_order(
        pos_id=pos_id,
        fiscal_void=fiscal_void,
        order_id=order_id,
        show_cancel_message=show_cancel_message,
        is_mobile=is_mobile,
        eft_processor=eft_processor,
        auth_data=auth_data,
        received_void_reason=received_void_reason,
        order_is_stored=True,
    )


def _void_order(
    pos_id,                 # type: str
    fiscal_void,            # type: str
    order_id,               # type: str
    show_cancel_message,    # type: bool
    is_mobile,              # type: bool
    eft_processor,          # type: str
    auth_data,              # type: str
    received_void_reason,   # type: str
    order_is_stored=False,  # type: Optional[bool]
):
    # type: (...) -> bool

    is_fiscal_void = fiscal_void.lower() == 'true'
    is_mobile = str(is_mobile).lower() == 'true'
    logger.debug("eft_processor voiding order {}".format(str(eft_processor)))
    void_type = 'VOID_PAID_ORDER' if is_fiscal_void else 'VOID_CURRENT_ORDER'
    void_type = 'VOID_STORED_ORDER' if order_is_stored else void_type

    try:
        model = sysactions.get_model(pos_id)

        auth_user = auth_data if auth_data is not None else None
        canceled_by = get_canceled_by(
            pos_id,
            model,
            auth_user,
        )

        if is_fiscal_void:
            order_id, void_reason = _void_fiscal_order(
                pos_id=pos_id,
                canceled_by=canceled_by,
                eft_processor=eft_processor,
                order_id=order_id,
            )
        elif received_void_reason is None:
            void_reason = _void_non_fiscal_order(
                pos_id=pos_id,
                order_id=order_id,
                model=model,
                canceled_by=canceled_by,
            )
        else:
            void_reason = get_fixed_void_reason(
                reason=received_void_reason,
                canceled_by=canceled_by,
                void_type=void_type,
            )

        _void_current_order(
            pos_id=pos_id,
            order_id=order_id,
            void_reason=void_reason,
        )

        _cancel_transactions(
            pos_id=pos_id,
            model=model,
            order_id=order_id,
            is_mobile=is_mobile,
            canceled_by=canceled_by,
        )

        justification_report(
            pos_id=pos_id,
            justification_type=void_type,
            justification=void_reason.get('VOID_REASON_DESCR'),
            authorized_by=canceled_by,
            order_id=order_id if order_id not in ["", "None", None] else None
        )

        send_order_canceled_event(
            pos_id=pos_id,
            order_id=order_id,
        )

        if show_cancel_message:
            sysactions.show_messagebox(pos_id, "$ORDER_VOIDED_WITH_SUCCESS")

        return True

    except sysactions.StopAction as _:
        return False

    except Exception as _: # noqa
        logger.exception("Error voiding order {}".format(str(order_id) if order_id else 'undefined'))
        if show_cancel_message:
            sysactions.show_messagebox(pos_id, "$ERROR_CANCEL_ORDER")

        return False


def _void_partner_order(
    pos_id,     # type: str
    order_id,   # type: str
):
    # type: (...) -> None

    remote_order_id = get_custom_property_value(
        pos_id=pos_id,
        custom_property="REMOTE_ORDER_ID",
        order_id=order_id,
    )

    if remote_order_id is not None:
        void_remote_order(
            pos_id=pos_id,
            remote_id=remote_order_id,
        )
        raise sysactions.StopAction()


def _void_current_order(
    pos_id,         # type: str
    order_id,       # type: str
    void_reason,    # type: Dict
):
    # type: (...) -> None

    for _ in range(VOID_ATTEMPTS):
        try:
            void_order(pos_id, order_id=order_id, void_reason=void_reason)
            break
        except Exception as _: # noqa
            logger.exception("Error voiding order")
            sleep(1)


def _cancel_transactions(
    pos_id,         # type: str
    model,          # type: eTree.XML
    order_id,       # type: str
    is_mobile,      # type: bool
    canceled_by,    # type: str
):
    # type: (...) -> None

    order_taker = sysactions.get_posot(model=model)
    order_picture = order_taker.orderPicture(
        posid=pos_id,
        orderid=order_id or ""
    )

    order_xml = eTree.XML(order_picture)
    order = order_xml.find("Order") if order_xml.find("Order") else order_xml

    clear_multiple_tenders(
        pos_id=pos_id,
        order=order,
        order_id=order_id,
        is_mobile=is_mobile,
        operator_id=canceled_by,
    )


def validate_eft_processor_order(
    eft_processor,      # type: str
    pos_id,             # type: str
    order_id,           # type: str
):
    # type: (...) -> None

    model = sysactions.get_model(posid=pos_id)
    order_taker = sysactions.get_posot(model=model)
    order_picture = order_taker.orderPicture(
        posid=pos_id,
        orderid=order_id,
    )

    order_xml = eTree.XML(order_picture)
    order = order_xml.find("Order") if order_xml.find("Order") else order_xml

    if not has_electronic_payment(order=order):
        return

    tef_status = check_tef_connectivity(pos_id=pos_id)
    if tef_status == "false":
        resp = show_message_options_dialog(
            pos_id=pos_id,
            buttons="$YES|$NO",
            title="$CONFIRMATION",
            message="$ORDER_CANCELLED_NO_REFUND",
        )
        if resp == 1:
            raise sysactions.StopAction()

    elif tef_status == EftConstants.INVALID_ELECTRONIC_PAYMENT_CONFIGURATION:
        sysactions.show_messagebox(
            posid=pos_id,
            message="$ERROR_CANCEL_ORDER_INVALID_POS_CONFIGURATION",
        )
        raise sysactions.StopAction()

    eft_is_valid = check_eft_is_valid(eft_processor=eft_processor)
    if not eft_is_valid:
        return

    tenders = get_electronic_tenders(order=order)
    for tender in tenders:
        is_electronic_payment = has_electronic_payment(
            order=None,
            tender_electronic_type=tender.get("electronicType"),
        )

        if is_electronic_payment:
            tender_detail = tender.get("detail")
            transaction_processor = tender_detail.get("TransactionProcessor", "")
            if transaction_processor.lower() != eft_processor.lower():
                message = "$ERROR_CANCEL_ORDER_ANOTHER_EFT_PROCESSOR|{}".format(transaction_processor)
                sysactions.show_messagebox(
                    posid=pos_id,
                    message=message,
                )
                raise sysactions.StopAction()


def _void_non_fiscal_order(
    pos_id,         # type: str
    order_id,       # type: str
    model,          # type: eTree.XML
    canceled_by,    # type: str
):
    # type: (...) -> Optional[Dict]

    _void_partner_order(
        pos_id=pos_id,
        order_id=order_id,
    )

    if is_stored_order(
        model=model,
        pos_id=pos_id,
        order_id=order_id,
    ):
        void_type = "VOID_STORED_ORDER"
    else:
        void_type = "VOID_CURRENT_ORDER"
    if must_require_void_reason(void_type):
        void_reason = get_void_reason_id(
            pos_id=pos_id,
            void_type=void_type,
            authentication_user=canceled_by,
        )
        if not void_reason:
            raise sysactions.StopAction()
    else:
        do_confirm_void(pos_id=pos_id)
        void_reason = get_fixed_void_reason(
            reason="USER_CANCELLATION",
            canceled_by=canceled_by,
            void_type=void_type,
        )

    if order_id in ["", None, "None"]:
        sysactions.check_current_order(pos_id, model)

    return void_reason


def _void_fiscal_order(
    pos_id,             # type: str
    canceled_by,        # type: str
    eft_processor,      # type: Optional[str]
    order_id=None,      # type: Optional[str]
):
    # type: (...) -> (str, dict)

    limit_datetime = datetime.utcnow() - timedelta(minutes=30)
    date_format = "%Y-%m-%dT%H:%M:%S"

    fiscalized_orders = OrderInteractor(pos_id=pos_id).get_fiscalized_orders(
        from_date=limit_datetime.strftime(date_format),
        pos_list=sysactions.get_poslist(),
    )

    order_options = []
    baseurl = "/mwapp/services/PosController/POS{}/?token=TK_POS_EXECUTEACTION&format=2&isBase64=true&payload="
    baseurl = baseurl.format(pos_id)
    order_descr = "{} - {} {}{:02} - {:05} - R${:.2f}"
    model = sysactions.get_model(posid=pos_id)
    pos = translate_pos_label(model=model)

    if order_id in ['', None]:
        for order in fiscalized_orders:
            fiscalization_date = datetime.strptime(order.fiscalization_date, date_format)
            descr = order_descr.format(
                convert_from_utf_to_localtime(utc=fiscalization_date).strftime('%H:%M'),
                order.pod_type,
                pos,
                int(order.originator_id[3:]),
                int(order.order_id),
                float(order.total_gross) + float(order.tip)
            )
            payload = base64.b64encode("\0".join(("doOrderPicture", pos_id, order.order_id)))
            url = baseurl + payload
            order_options.append((order.order_id, descr, url))

    if len(order_options) > 0:
        order_id = _get_order_id_by_dialog(
            pos_id=pos_id,
            options=order_options,

        )

    if not order_id:
        sysactions.show_messagebox(
            posid=pos_id,
            message="$NO_ORDER_FOUND_TO_CANCEL",
        )
        raise sysactions.StopAction()

    validate_eft_processor_order(
        eft_processor=eft_processor,
        pos_id=pos_id,
        order_id=order_id,
    )

    _void_partner_order(
        pos_id=pos_id,
        order_id=order_id,
    )

    void_type = "VOID_PAID_ORDER"
    if must_require_void_reason(void_on=void_type):
        void_reason = get_void_reason_id(
            pos_id=pos_id,
            void_type=void_type,
            authentication_user=canceled_by,
        )
        if not void_reason:
            raise sysactions.StopAction()
    else:
        do_confirm_void(pos_id=pos_id)
        void_reason = get_fixed_void_reason(
            reason="USER_FISCAL_CANCELLATION",
            canceled_by=canceled_by,
            void_type=void_type,
        )
        if not void_reason:
            raise sysactions.StopAction()

    void_fiscal_order(
        pos_id=int(pos_id),
        order_id=int(order_id),
        justification=void_reason.get("VOID_REASON_DESCR", None),
    )

    return order_id, void_reason


def _get_order_id_by_dialog(
    pos_id,         # type: str
    options,  # type: List
):
    # type: (...) -> str

    selected = sysactions.show_order_preview(
        posid=pos_id,
        orders=options,
        title="$SELECT_AN_ORDER_TO_CANCEL",
        buttons="CANCEL|OK",
        onthefly=True
    )
    if (selected is None) or (selected[0] == "0"):
        raise sysactions.StopAction()

    return selected[1]



import os

from enum import Enum

from systools import sys_parsefilepath


class SystemPaths(Enum):

    data_path = sys_parsefilepath("{HVDATADIR}")
    genesis_data_path = os.path.join(data_path, "../../genesis/data")
    genesis_server_path = os.path.join(data_path, "../../genesis/data/server")
    genesis_databases_path = os.path.join(data_path, "../../genesis/data/server/databases")
    genesis_bundles_path = os.path.join(data_path, "../../genesis/data/server/bundles")
    static_configuration_folder = os.path.join(data_path, "../../genesis", "static")



from production.box import ProdStateFilterBox
from production.treebuilder import BoxConfiguration

from ._BoxBuilder import BoxBuilder


class ProdStateFilterBoxBuilder(BoxBuilder):
    def __init__(self, product_repository, loggers):
        super(ProdStateFilterBoxBuilder, self).__init__(loggers)
        self.product_repository = product_repository

    def build(self, config):
        # type: (BoxConfiguration) -> ProdStateFilterBox
        return ProdStateFilterBox(config.name,
                                  config.sons,
                                  self.get_extra(config, "AllowedProdStates", []),
                                  self.get_extra(config, "ExcludedProdStates", []),
                                  self.get_logger(config))



import json

from messagebus import (
    AckMessage,
    Message,
    MessageBus,
    InternalErrorMessage,
)
from messageprocessorutil import UuidMessageProcessor
from typing import (
    Any,
    Optional,
)

from devicemanager.interactor.get_used_service import GetUsedServiceInteractor
from devicemanager.interactor.get_used_service.dto import GetUsedServiceRequest


class GetUsedServiceProcessor(UuidMessageProcessor):
    def __init__(
        self,
        get_used_service_interactor,    # type: GetUsedServiceInteractor
    ):
        # type: (...) -> None

        super(GetUsedServiceProcessor, self).__init__()
        self.get_used_service_interactor = get_used_service_interactor

    def parse_data(
        self,           # type: GetUsedServiceProcessor
        data,           # type: str
        event_type,     # type: str
    ):
        # type: (...) -> GetUsedServiceRequest

        request_json = json.loads(data)
        self._validate_request(request_json)
        return GetUsedServiceRequest(
            pos_id=request_json.get("posId", None),
            device_type=request_json["deviceType"],
            sector=request_json.get("sector", None),
            kds_id=request_json.get("kdsId", None),
            brand=request_json.get("brand", None),
        )

    def call_business(
        self,       # type: GetUsedServiceProcessor
        request,    # type: GetUsedServiceRequest
    ):
        # type: (...) -> str

        return self.get_used_service_interactor.get_used_service(request=request)

    def format_response(
        self,           # type: GetUsedServiceProcessor
        message_bus,    # type: MessageBus
        message,        # type: Message
        event,          # type: Optional[Any]
        request,        # type: GetUsedServiceRequest
        result,         # type: str
    ):
        # type: (...) -> None

        data = result.encode("utf-8") if result is not None else None
        message_bus.reply_message(
            message=message,
            reply_message=AckMessage(data=data),
        )

    def format_exception(
        self,           # type: GetUsedServiceProcessor
        message_bus,    # type: MessageBus
        message,        # type: Message
        event,          # type: Optional[Any]
        input_obj,      # type: GetUsedServiceRequest
        exception,      # type: Exception
    ):
        # type: (...) -> None

        message_bus.reply_message(
            message=message,
            reply_message=InternalErrorMessage(ex=exception),
        )

    def format_parse_exception(
        self,           # type: GetUsedServiceProcessor
        message_bus,    # type: MessageBus
        message,        # type: Message
        event,          # type: Optional[Any]
        data,           # type: Any
        exception,      # type: Exception
    ):
        # type: (...) -> None

        message_bus.reply_message(
            message=message,
            reply_message=InternalErrorMessage(ex=exception),
        )

    @staticmethod
    def _validate_request(
        request_json,   # type: dict
    ):
        # type: (...) -> None

        if "deviceType" not in request_json:
            raise Exception("Invalid request json: deviceType key not sent")
        if "posId" not in request_json and "kdsId" not in request_json:
            raise Exception("Invalid request json: posId nor kdsId sent")



from _SaleChallenge import SaleChallenge
from ._Goals import Goals
from ._StoreGoals import StoreGoals
from ._ItemGoals import ItemGoals
from ._OperatorGoals import OperatorGoals
from ._GoalsCache import GoalsCache
from ._Order import Order
from ._OrderItem import OrderItem
from ._PartCode import PartCode
from _DailyGoalsException import (
    InvalidRequestException, 
    DailyGoalsExistsException, 
    DailyGoalsNonExistsException,
    InsertDailyGoalsException, 
    UpdateDailyGoalsException, 
    DeleteDailyGoalsException, 
    NotCacheGoals,
    GetSalesChallengeException, 
    InsertSalesChallengeException, 
    DeleteSalesChallengeException,
    UpdateSalesChallengeException,
    GetPaidOrdersException, 
    GetOrderItemsException,
)



""" Python Character Mapping Codec generated from 'VENDORS/MICSFT/PC/CP860.TXT' with gencodec.py.
"""

import codecs

from typing import (
    Union,
    Optional,
)


def custom_encode(
    text,             # type: Union[str, unicode]
    errors='strict',  # type: Optional[str]
):
    return codecs.charmap_encode(
        text,
        errors,
        encoding_map
    )[0]


encoding_map = {
    0x0000: 0x0000,  # NULL
    0x0001: 0x0001,  # START OF HEADING
    0x0002: 0x0002,  # START OF TEXT
    0x0003: 0x0003,  # END OF TEXT
    0x0004: 0x0004,  # END OF TRANSMISSION
    0x0005: 0x0005,  # ENQUIRY
    0x0006: 0x0006,  # ACKNOWLEDGE
    0x0007: 0x0007,  # BELL
    0x0008: 0x0008,  # BACKSPACE
    0x0009: 0x0009,  # HORIZONTAL TABULATION
    0x000a: 0x000a,  # LINE FEED
    0x000b: 0x000b,  # VERTICAL TABULATION
    0x000c: 0x000c,  # FORM FEED
    0x000d: 0x000d,  # CARRIAGE RETURN
    0x000e: 0x000e,  # SHIFT OUT
    0x000f: 0x000f,  # SHIFT IN
    0x0010: 0x0010,  # DATA LINK ESCAPE
    0x0011: 0x0011,  # DEVICE CONTROL ONE
    0x0012: 0x0012,  # DEVICE CONTROL TWO
    0x0013: 0x0013,  # DEVICE CONTROL THREE
    0x0014: 0x0014,  # DEVICE CONTROL FOUR
    0x0015: 0x0015,  # NEGATIVE ACKNOWLEDGE
    0x0016: 0x0016,  # SYNCHRONOUS IDLE
    0x0017: 0x0017,  # END OF TRANSMISSION BLOCK
    0x0018: 0x0018,  # CANCEL
    0x0019: 0x0019,  # END OF MEDIUM
    0x001a: 0x001a,  # SUBSTITUTE
    0x001b: 0x001b,  # ESCAPE
    0x001c: 0x001c,  # FILE SEPARATOR
    0x001d: 0x001d,  # GROUP SEPARATOR
    0x001e: 0x001e,  # RECORD SEPARATOR
    0x001f: 0x001f,  # UNIT SEPARATOR
    0x0020: 0x0020,  # SPACE
    0x0021: 0x0021,  # EXCLAMATION MARK
    0x0022: 0x0022,  # QUOTATION MARK
    0x0023: 0x0023,  # NUMBER SIGN
    0x0024: 0x0024,  # DOLLAR SIGN
    0x0025: 0x0025,  # PERCENT SIGN
    0x0026: 0x0026,  # AMPERSAND
    0x0027: 0x0027,  # APOSTROPHE
    0x0028: 0x0028,  # LEFT PARENTHESIS
    0x0029: 0x0029,  # RIGHT PARENTHESIS
    0x002a: 0x002a,  # ASTERISK
    0x002b: 0x002b,  # PLUS SIGN
    0x002c: 0x002c,  # COMMA
    0x002d: 0x002d,  # HYPHEN-MINUS
    0x002e: 0x002e,  # FULL STOP
    0x002f: 0x002f,  # SOLIDUS
    0x0030: 0x0030,  # DIGIT ZERO
    0x0031: 0x0031,  # DIGIT ONE
    0x0032: 0x0032,  # DIGIT TWO
    0x0033: 0x0033,  # DIGIT THREE
    0x0034: 0x0034,  # DIGIT FOUR
    0x0035: 0x0035,  # DIGIT FIVE
    0x0036: 0x0036,  # DIGIT SIX
    0x0037: 0x0037,  # DIGIT SEVEN
    0x0038: 0x0038,  # DIGIT EIGHT
    0x0039: 0x0039,  # DIGIT NINE
    0x003a: 0x003a,  # COLON
    0x003b: 0x003b,  # SEMICOLON
    0x003c: 0x003c,  # LESS-THAN SIGN
    0x003d: 0x003d,  # EQUALS SIGN
    0x003e: 0x003e,  # GREATER-THAN SIGN
    0x003f: 0x003f,  # QUESTION MARK
    0x0040: 0x0040,  # COMMERCIAL AT
    0x0041: 0x0041,  # LATIN CAPITAL LETTER A
    0x0042: 0x0042,  # LATIN CAPITAL LETTER B
    0x0043: 0x0043,  # LATIN CAPITAL LETTER C
    0x0044: 0x0044,  # LATIN CAPITAL LETTER D
    0x0045: 0x0045,  # LATIN CAPITAL LETTER E
    0x0046: 0x0046,  # LATIN CAPITAL LETTER F
    0x0047: 0x0047,  # LATIN CAPITAL LETTER G
    0x0048: 0x0048,  # LATIN CAPITAL LETTER H
    0x0049: 0x0049,  # LATIN CAPITAL LETTER I
    0x004a: 0x004a,  # LATIN CAPITAL LETTER J
    0x004b: 0x004b,  # LATIN CAPITAL LETTER K
    0x004c: 0x004c,  # LATIN CAPITAL LETTER L
    0x004d: 0x004d,  # LATIN CAPITAL LETTER M
    0x004e: 0x004e,  # LATIN CAPITAL LETTER N
    0x004f: 0x004f,  # LATIN CAPITAL LETTER O
    0x0050: 0x0050,  # LATIN CAPITAL LETTER P
    0x0051: 0x0051,  # LATIN CAPITAL LETTER Q
    0x0052: 0x0052,  # LATIN CAPITAL LETTER R
    0x0053: 0x0053,  # LATIN CAPITAL LETTER S
    0x0054: 0x0054,  # LATIN CAPITAL LETTER T
    0x0055: 0x0055,  # LATIN CAPITAL LETTER U
    0x0056: 0x0056,  # LATIN CAPITAL LETTER V
    0x0057: 0x0057,  # LATIN CAPITAL LETTER W
    0x0058: 0x0058,  # LATIN CAPITAL LETTER X
    0x0059: 0x0059,  # LATIN CAPITAL LETTER Y
    0x005a: 0x005a,  # LATIN CAPITAL LETTER Z
    0x005b: 0x005b,  # LEFT SQUARE BRACKET
    0x005c: 0x005c,  # REVERSE SOLIDUS
    0x005d: 0x005d,  # RIGHT SQUARE BRACKET
    0x005e: 0x005e,  # CIRCUMFLEX ACCENT
    0x005f: 0x005f,  # LOW LINE
    0x0060: 0x0060,  # GRAVE ACCENT
    0x0061: 0x0061,  # LATIN SMALL LETTER A
    0x0062: 0x0062,  # LATIN SMALL LETTER B
    0x0063: 0x0063,  # LATIN SMALL LETTER C
    0x0064: 0x0064,  # LATIN SMALL LETTER D
    0x0065: 0x0065,  # LATIN SMALL LETTER E
    0x0066: 0x0066,  # LATIN SMALL LETTER F
    0x0067: 0x0067,  # LATIN SMALL LETTER G
    0x0068: 0x0068,  # LATIN SMALL LETTER H
    0x0069: 0x0069,  # LATIN SMALL LETTER I
    0x006a: 0x006a,  # LATIN SMALL LETTER J
    0x006b: 0x006b,  # LATIN SMALL LETTER K
    0x006c: 0x006c,  # LATIN SMALL LETTER L
    0x006d: 0x006d,  # LATIN SMALL LETTER M
    0x006e: 0x006e,  # LATIN SMALL LETTER N
    0x006f: 0x006f,  # LATIN SMALL LETTER O
    0x0070: 0x0070,  # LATIN SMALL LETTER P
    0x0071: 0x0071,  # LATIN SMALL LETTER Q
    0x0072: 0x0072,  # LATIN SMALL LETTER R
    0x0073: 0x0073,  # LATIN SMALL LETTER S
    0x0074: 0x0074,  # LATIN SMALL LETTER T
    0x0075: 0x0075,  # LATIN SMALL LETTER U
    0x0076: 0x0076,  # LATIN SMALL LETTER V
    0x0077: 0x0077,  # LATIN SMALL LETTER W
    0x0078: 0x0078,  # LATIN SMALL LETTER X
    0x0079: 0x0079,  # LATIN SMALL LETTER Y
    0x007a: 0x007a,  # LATIN SMALL LETTER Z
    0x007b: 0x007b,  # LEFT CURLY BRACKET
    0x007c: 0x007c,  # VERTICAL LINE
    0x007d: 0x007d,  # RIGHT CURLY BRACKET
    0x007e: 0x007e,  # TILDE
    0x007f: 0x007f,  # DELETE
    0x00a0: 0x00ff,  # NO-BREAK SPACE
    0x00a1: 0x00ad,  # INVERTED EXCLAMATION MARK
    0x00a3: 0x009c,  # POUND SIGN
    0x00ab: 0x00ae,  # LEFT-POINTING DOUBLE ANGLE QUOTATION MARK
    0x00ac: 0x00aa,  # NOT SIGN
    0x00b0: 0x00f8,  # DEGREE SIGN
    0x00b1: 0x00f1,  # PLUS-MINUS SIGN
    0x00b2: 0x00fd,  # SUPERSCRIPT TWO
    0x00b5: 0x00e6,  # MICRO SIGN
    0x00b7: 0x00fa,  # MIDDLE DOT
    0x00bb: 0x00af,  # RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK
    0x00bc: 0x00ac,  # VULGAR FRACTION ONE QUARTER
    0x00bd: 0x00ab,  # VULGAR FRACTION ONE HALF
    0x00bf: 0x00a8,  # INVERTED QUESTION MARK
    0x00c1: 0x00a4,  # LATIN CAPITAL LETTER A WITH ACUTE
    0x00c4: 0x008e,  # LATIN CAPITAL LETTER A WITH DIAERESIS
    0x00c3: 0x008e,  # LATIN CAPITAL LETTER A WITH TILDE
    0x00c5: 0x008f,  # LATIN CAPITAL LETTER A WITH RING ABOVE
    0x00c6: 0x0092,  # LATIN CAPITAL LIGATURE AE
    0x00c7: 0x0080,  # LATIN CAPITAL LETTER C WITH CEDILLA
    0x00c9: 0x0090,  # LATIN CAPITAL LETTER E WITH ACUTE
    0x00cd: 0x00a5,  # LATIN CAPITAL LETTER I WITH ACUTE
    0x00d0: 0x008b,  # LATIN CAPITAL LETTER ETH
    0x00d3: 0x00a6,  # LATIN CAPITAL LETTER O WITH ACUTE
    0x00d6: 0x0099,  # LATIN CAPITAL LETTER O WITH DIAERESIS
    0x00d8: 0x009d,  # LATIN CAPITAL LETTER O WITH STROKE
    0x00da: 0x00a7,  # LATIN CAPITAL LETTER U WITH ACUTE
    0x00dc: 0x009a,  # LATIN CAPITAL LETTER U WITH DIAERESIS
    0x00dd: 0x0097,  # LATIN CAPITAL LETTER Y WITH ACUTE
    0x00de: 0x008d,  # LATIN CAPITAL LETTER THORN
    0x00df: 0x00e1,  # LATIN SMALL LETTER SHARP S
    0x00e0: 0x0085,  # LATIN SMALL LETTER A WITH GRAVE
    0x00e1: 0x00a0,  # LATIN SMALL LETTER A WITH ACUTE
    0x00e2: 0x0083,  # LATIN SMALL LETTER A WITH CIRCUMFLEX
    0x00e4: 0x0084,  # LATIN SMALL LETTER A WITH DIAERESIS
    0x00e5: 0x0086,  # LATIN SMALL LETTER A WITH RING ABOVE
    0x00e6: 0x0091,  # LATIN SMALL LIGATURE AE
    0x00e7: 0x0087,  # LATIN SMALL LETTER C WITH CEDILLA
    0x00e8: 0x008a,  # LATIN SMALL LETTER E WITH GRAVE
    0x00e9: 0x0082,  # LATIN SMALL LETTER E WITH ACUTE
    0x00ea: 0x0088,  # LATIN SMALL LETTER E WITH CIRCUMFLEX
    0x00eb: 0x0089,  # LATIN SMALL LETTER E WITH DIAERESIS
    0x00ed: 0x00a1,  # LATIN SMALL LETTER I WITH ACUTE
    0x00f0: 0x008c,  # LATIN SMALL LETTER ETH
    0x00f3: 0x00a2,  # LATIN SMALL LETTER O WITH ACUTE
    0x00f4: 0x0093,  # LATIN SMALL LETTER O WITH CIRCUMFLEX
    0x00f6: 0x0094,  # LATIN SMALL LETTER O WITH DIAERESIS
    0x00f7: 0x00f6,  # DIVISION SIGN
    0x00f8: 0x009b,  # LATIN SMALL LETTER O WITH STROKE
    0x00fa: 0x00a3,  # LATIN SMALL LETTER U WITH ACUTE
    0x00fc: 0x0081,  # LATIN SMALL LETTER U WITH DIAERESIS
    0x00fd: 0x0098,  # LATIN SMALL LETTER Y WITH ACUTE
    0x00fe: 0x0095,  # LATIN SMALL LETTER THORN
    0x0192: 0x009f,  # LATIN SMALL LETTER F WITH HOOK
    0x0393: 0x00e2,  # GREEK CAPITAL LETTER GAMMA
    0x0398: 0x00e9,  # GREEK CAPITAL LETTER THETA
    0x03a3: 0x00e4,  # GREEK CAPITAL LETTER SIGMA
    0x03a6: 0x00e8,  # GREEK CAPITAL LETTER PHI
    0x03a9: 0x00ea,  # GREEK CAPITAL LETTER OMEGA
    0x03b1: 0x00e0,  # GREEK SMALL LETTER ALPHA
    0x03b4: 0x00eb,  # GREEK SMALL LETTER DELTA
    0x03b5: 0x00ee,  # GREEK SMALL LETTER EPSILON
    0x03c0: 0x00e3,  # GREEK SMALL LETTER PI
    0x03c3: 0x00e5,  # GREEK SMALL LETTER SIGMA
    0x03c4: 0x00e7,  # GREEK SMALL LETTER TAU
    0x03c6: 0x00ed,  # GREEK SMALL LETTER PHI
    0x0404: 0x00A4,  # CYRILLIC CAPITAL LETTER UKRAINIAN
    0x207f: 0x00fc,  # SUPERSCRIPT LATIN SMALL LETTER N
    0x20a7: 0x009e,  # PESETA SIGN
    0x2219: 0x00f9,  # BULLET OPERATOR
    0x221a: 0x00fb,  # SQUARE ROOT
    0x221e: 0x00ec,  # INFINITY
    0x2229: 0x00ef,  # INTERSECTION
    0x2248: 0x00f7,  # ALMOST EQUAL TO
    0x2261: 0x00f0,  # IDENTICAL TO
    0x2264: 0x00f3,  # LESS-THAN OR EQUAL TO
    0x2265: 0x00f2,  # GREATER-THAN OR EQUAL TO
    0x2310: 0x00a9,  # REVERSED NOT SIGN
    0x2320: 0x00f4,  # TOP HALF INTEGRAL
    0x2321: 0x00f5,  # BOTTOM HALF INTEGRAL
    0x2500: 0x00c4,  # BOX DRAWINGS LIGHT HORIZONTAL
    0x2502: 0x00b3,  # BOX DRAWINGS LIGHT VERTICAL
    0x250c: 0x00da,  # BOX DRAWINGS LIGHT DOWN AND RIGHT
    0x2510: 0x00bf,  # BOX DRAWINGS LIGHT DOWN AND LEFT
    0x2514: 0x00c0,  # BOX DRAWINGS LIGHT UP AND RIGHT
    0x2518: 0x00d9,  # BOX DRAWINGS LIGHT UP AND LEFT
    0x251c: 0x00c3,  # BOX DRAWINGS LIGHT VERTICAL AND RIGHT
    0x2524: 0x00b4,  # BOX DRAWINGS LIGHT VERTICAL AND LEFT
    0x252c: 0x00c2,  # BOX DRAWINGS LIGHT DOWN AND HORIZONTAL
    0x2534: 0x00c1,  # BOX DRAWINGS LIGHT UP AND HORIZONTAL
    0x253c: 0x00c5,  # BOX DRAWINGS LIGHT VERTICAL AND HORIZONTAL
    0x2550: 0x00cd,  # BOX DRAWINGS DOUBLE HORIZONTAL
    0x2551: 0x00ba,  # BOX DRAWINGS DOUBLE VERTICAL
    0x2552: 0x00d5,  # BOX DRAWINGS DOWN SINGLE AND RIGHT DOUBLE
    0x2553: 0x00d6,  # BOX DRAWINGS DOWN DOUBLE AND RIGHT SINGLE
    0x2554: 0x00c9,  # BOX DRAWINGS DOUBLE DOWN AND RIGHT
    0x2555: 0x00b8,  # BOX DRAWINGS DOWN SINGLE AND LEFT DOUBLE
    0x2556: 0x00b7,  # BOX DRAWINGS DOWN DOUBLE AND LEFT SINGLE
    0x2557: 0x00bb,  # BOX DRAWINGS DOUBLE DOWN AND LEFT
    0x2558: 0x00d4,  # BOX DRAWINGS UP SINGLE AND RIGHT DOUBLE
    0x2559: 0x00d3,  # BOX DRAWINGS UP DOUBLE AND RIGHT SINGLE
    0x255a: 0x00c8,  # BOX DRAWINGS DOUBLE UP AND RIGHT
    0x255b: 0x00be,  # BOX DRAWINGS UP SINGLE AND LEFT DOUBLE
    0x255c: 0x00bd,  # BOX DRAWINGS UP DOUBLE AND LEFT SINGLE
    0x255d: 0x00bc,  # BOX DRAWINGS DOUBLE UP AND LEFT
    0x255e: 0x00c6,  # BOX DRAWINGS VERTICAL SINGLE AND RIGHT DOUBLE
    0x255f: 0x00c7,  # BOX DRAWINGS VERTICAL DOUBLE AND RIGHT SINGLE
    0x2560: 0x00cc,  # BOX DRAWINGS DOUBLE VERTICAL AND RIGHT
    0x2561: 0x00b5,  # BOX DRAWINGS VERTICAL SINGLE AND LEFT DOUBLE
    0x2562: 0x00b6,  # BOX DRAWINGS VERTICAL DOUBLE AND LEFT SINGLE
    0x2563: 0x00b9,  # BOX DRAWINGS DOUBLE VERTICAL AND LEFT
    0x2564: 0x00d1,  # BOX DRAWINGS DOWN SINGLE AND HORIZONTAL DOUBLE
    0x2565: 0x00d2,  # BOX DRAWINGS DOWN DOUBLE AND HORIZONTAL SINGLE
    0x2566: 0x00cb,  # BOX DRAWINGS DOUBLE DOWN AND HORIZONTAL
    0x2567: 0x00cf,  # BOX DRAWINGS UP SINGLE AND HORIZONTAL DOUBLE
    0x2568: 0x00d0,  # BOX DRAWINGS UP DOUBLE AND HORIZONTAL SINGLE
    0x2569: 0x00ca,  # BOX DRAWINGS DOUBLE UP AND HORIZONTAL
    0x256a: 0x00d8,  # BOX DRAWINGS VERTICAL SINGLE AND HORIZONTAL DOUBLE
    0x256b: 0x00d7,  # BOX DRAWINGS VERTICAL DOUBLE AND HORIZONTAL SINGLE
    0x256c: 0x00ce,  # BOX DRAWINGS DOUBLE VERTICAL AND HORIZONTAL
    0x2580: 0x00df,  # UPPER HALF BLOCK
    0x2584: 0x00dc,  # LOWER HALF BLOCK
    0x2588: 0x00db,  # FULL BLOCK
    0x258c: 0x00dd,  # LEFT HALF BLOCK
    0x2590: 0x00de,  # RIGHT HALF BLOCK
    0x2591: 0x00b0,  # LIGHT SHADE
    0x2592: 0x00b1,  # MEDIUM SHADE
    0x2593: 0x00b2,  # DARK SHADE
    0x25a0: 0x00fe,  # BLACK SQUARE

    # Manualy
    0x00fb: 0x0096,  # LATIN SMALL LETTER U WITH CIRCUMFLEX
    0x00db: 0x0096,  # LATIN CAPITAL LETTER U WITH CIRCUMFLEX
    0x20ac: 0x00d5,  # EURO SIGN
    0x201c: 0x00f2,  # LEFT DOUBLE QUOTATION MARK
    0x201d: 0x00a6,  # RIGHT DOUBLE QUOTATION MARK
    0x2013: 0x002d,  # BIG DASH

    # CP860
    0x00a2: 0x009b,  # CENT SIGN
    0x00aa: 0x00a6,  # FEMININE ORDINAL INDICATOR
    0x00ba: 0x00a7,  # MASCULINE ORDINAL INDICATOR
    0x00c0: 0x0091,  # LATIN CAPITAL LETTER A WITH GRAVE
    0x00c2: 0x008f,  # LATIN CAPITAL LETTER A WITH CIRCUMFLEX
    0x00c8: 0x0092,  # LATIN CAPITAL LETTER E WITH GRAVE
    0x00ca: 0x0089,  # LATIN CAPITAL LETTER E WITH CIRCUMFLEX
    0x00cc: 0x0098,  # LATIN CAPITAL LETTER I WITH GRAVE
    0x00d1: 0x00a5,  # LATIN CAPITAL LETTER N WITH TILDE
    0x00d2: 0x00a9,  # LATIN CAPITAL LETTER O WITH GRAVE
    0x00d4: 0x008c,  # LATIN CAPITAL LETTER O WITH CIRCUMFLEX
    0x00d5: 0x0099,  # LATIN CAPITAL LETTER O WITH TILDE
    0x00d9: 0x009d,  # LATIN CAPITAL LETTER U WITH GRAVE
    0x00e3: 0x0084,  # LATIN SMALL LETTER A WITH TILDE
    0x00ec: 0x008d,  # LATIN SMALL LETTER I WITH GRAVE
    0x00f1: 0x00a4,  # LATIN SMALL LETTER N WITH TILDE
    0x00f2: 0x0095,  # LATIN SMALL LETTER O WITH GRAVE
    0x00f5: 0x0094,  # LATIN SMALL LETTER O WITH TILDE
    0x00f9: 0x0097,  # LATIN SMALL LETTER U WITH GRAVE

}



# -*- coding: utf-8 -*-
import os
import qrcode

from typing import Optional
from constants import *
from image import EscposImage
from PIL import Image, ImageChops, ImageOps, ImageFont, ImageDraw



def trim(im):
    bg = Image.new(im.mode, im.size, im.getpixel((0, 0)))
    diff = ImageChops.difference(im, bg)
    diff = ImageChops.add(diff, diff, 2.0, -100)
    bbox = diff.getbbox()
    if bbox:
        return im.crop(bbox)


def image(img_source, high_density_vertical=True, high_density_horizontal=True, impl="bitImageRaster",
          fragment_height=1024):
    """ Print an image

    You can select whether the printer should print in high density or not. The default value is high density.
    When printing in low density, the image will be stretched.

    Esc/Pos supplies several commands for printing. This function supports three of them. Please try to vary the
    implementations if you have any problems. For example the printer `IT80-002` will have trouble aligning
    images that are not printed in Column-mode.

    The available printing implementations are:

        * `bitImageRaster`: prints with the `GS v 0`-command
        * `graphics`: prints with the `GS ( L`-command
        * `bitImageColumn`: prints with the `ESC *`-command

    :param img_source: PIL image or filename to load: `jpg`, `gif`, `png` or `bmp`
    :param high_density_vertical: print in high density in vertical direction *default:* True
    :param high_density_horizontal: print in high density in horizontal direction *default:* True
    :param impl: choose image printing mode between `bitImageRaster`, `graphics` or `bitImageColumn`
    :param fragment_height: Images larger than this will be split into multiple fragments *default:* 1024

    """
    # with open("\\\\.\\COM8", "w") as prnt:
    im = EscposImage(img_source)
    prnt = ""
    if im.height > fragment_height:
        fragments = im.split(fragment_height)
        for fragment in fragments:
            image(fragment,
                  high_density_vertical=high_density_vertical,
                  high_density_horizontal=high_density_horizontal,
                  impl=impl,
                  fragment_height=fragment_height)
        return

    if impl == "bitImageRaster":
        # GS v 0, raster format bit image
        density_byte = (0 if high_density_horizontal else 1) + (0 if high_density_vertical else 2)
        header = (
                GS
                + b"v0"
                + six.int2byte(density_byte)
                + _int_low_high(im.width_bytes, 2)
                + _int_low_high(im.height, 2)
        )
        prnt += (header + im.to_raster_format())
        return prnt

    if impl == "graphics":
        # GS ( L raster format graphics
        img_header = _int_low_high(im.width, 2) + _int_low_high(im.height, 2)
        tone = b'0'
        colors = b'1'
        ym = six.int2byte(1 if high_density_vertical else 2)
        xm = six.int2byte(1 if high_density_horizontal else 2)
        header = tone + xm + ym + colors + img_header
        raster_data = im.to_raster_format()
        return _image_send_graphics_data(b'0', b'p', header + raster_data)
        # _image_send_graphics_data(b'0', b'2', b'')

    if impl == "bitImageColumn":
        # ESC *, column format bit image
        density_byte = (1 if high_density_horizontal else 0) + (32 if high_density_vertical else 0)
        header = ESC + b"*" + six.int2byte(density_byte) + _int_low_high(im.width, 2)
        outp = []  # Adjust line-feed size
        for blob in im.to_column_format(high_density_vertical):
            outp.append(header + blob + b"\n")
        outp.append(ESC + b"2")  # Reset line-feed size
        return b''.join(outp)


def qr(content, ec=QR_ECLEVEL_L, size=3, model=QR_MODEL_2, native=False, is_fake=False):
    """ Print QR Code for the provided string

    :param content: The content of the code. Numeric data will be more efficiently compacted.
    :param ec: Error-correction level to use. One of QR_ECLEVEL_L (default), QR_ECLEVEL_M, QR_ECLEVEL_Q or
        QR_ECLEVEL_H.
        Higher error correction results in a less compact code.
    :param size: Pixel size to use. Must be 1-16 (default 3)
    :param model: QR code model to use. Must be one of QR_MODEL_1, QR_MODEL_2 (default) or QR_MICRO (not supported
        by all printers).
    :param native: True to render the code on the printer, False to render the code as an image and send it to the
        printer (Default)
    :param is_fake: True to render the code on size 8px else using size
    """
    size = 8 if is_fake else size
    # Basic validation
    if ec not in [QR_ECLEVEL_L, QR_ECLEVEL_M, QR_ECLEVEL_H, QR_ECLEVEL_Q]:
        raise ValueError("Invalid error correction level")
    if not 1 <= size <= 16:
        raise ValueError("Invalid block size (must be 1-16)")
    if model not in [QR_MODEL_1, QR_MODEL_2, QR_MICRO]:
        raise ValueError("Invalid QR model (must be one of QR_MODEL_1, QR_MODEL_2, QR_MICRO)")
    if content == "":
        # Handle edge case by printing nothing.
        return
    if not native:
        # Map ESC/POS error correction levels to python 'qrcode' library constant and render to an image
        if model != QR_MODEL_2:
            raise ValueError("Invalid QR model for qrlib rendering (must be QR_MODEL_2)")
        python_qr_ec = {
            QR_ECLEVEL_H: qrcode.constants.ERROR_CORRECT_H,
            QR_ECLEVEL_L: qrcode.constants.ERROR_CORRECT_L,
            QR_ECLEVEL_M: qrcode.constants.ERROR_CORRECT_M,
            QR_ECLEVEL_Q: qrcode.constants.ERROR_CORRECT_Q
        }
        qr_code = qrcode.QRCode(version=None, box_size=size, border=1, error_correction=python_qr_ec[ec])
        qr_code.add_data(content)
        qr_code.make(fit=True)
        qr_img = qr_code.make_image()

        im = qr_img._img.convert("RGB")
        return im


def insert_text_on_right_image(im, text):
    data_dir = os.environ["HVDATADIR"]

    fontsize = 18
    font = ImageFont.truetype(
        os.path.join(data_dir, "../../src/fiscalwrapper/src/fonts", "arial.ttf"),
        fontsize,
        encoding='utf-8'
    )
    img = Image.new("RGB", (int(560 - im.width), im.height), (255, 255, 255))
    draw = ImageDraw.Draw(img)
    i = 0
    for line in text:
        y = int(i * img.height / len(text))
        draw.text((0, y), "  {}".format(line).decode('utf-8'), (0, 0, 0), font=font)
        i += 1

    def get_concat_h(im1, im2):
        dst = Image.new('RGB', (im1.width + im2.width, im1.height))
        dst.paste(im1, (0, 0))
        dst.paste(im2, (im1.width, 0))
        return dst

    return image(get_concat_h(im, img))


def insert_space(string, length):
    return ' '.join(string[i:i + length] for i in xrange(0, len(string), length))


def barcode(code, one_line=True):
    import barcode
    from PIL import Image
    from barcode.writer import ImageWriter
    wrt = ImageWriter(print_text=False)

    if not one_line:
        # Return barcode in two lines
        code128_1 = barcode.Code128(code[:22], writer=wrt)
        code128_1.save('imagem')
        code128_2 = barcode.Code128(code[22:], writer=wrt)
        code128_2.save('imagem2')

        with Image.open('imagem.png') as img1:
            with Image.open('imagem2.png') as img2:
                with img1.resize((img1.width, int(img1.height * 0.33))) as img3:
                    with img2.resize((img2.width, int(img2.height * 0.33))) as img4:
                        with img3.convert("RGB") as pilimg1:
                            with img4.convert("RGB") as pilimg2:
                                return image(pilimg1), image(pilimg2)
    else:
        # Return barcode in one line
        code128_1 = barcode.Code128(code[:], writer=wrt)
        code128_1.save('imagem')
        with Image.open('imagem.png') as img1:
            with trim(img1) as img2:  # Trim image borders
                with img2.resize((int(img2.width * 0.78), int(
                        img2.height * 0.5))) as img3:  # Magic numbers so that image fits in Epson TMT-T88IV using a total of 56 characters
                    with img3.convert("RGB") as pilimg1:
                        return image(pilimg1), ""


def get_logo(is_larger_paper=True):
    # type: (bool) -> Image

    image_logo = _logo_path(is_larger_paper)
    if image_logo is None:
        return None

    base_width = 100 if is_larger_paper else 180
    with Image.open(image_logo) as image_opened:
        width_percent = (base_width / float(image_opened.size[0]))
        height_size = int((float(image_opened.size[1]) * float(width_percent)))
        with image_opened.resize((base_width, height_size)) as image_resize:
            with image_resize.convert("RGBA") as image_converted:
                return image_converted


def _logo_path(is_larger_paper):
    # type: (bool) -> Optional[str]

    data_dir = os.environ["HVDATADIR"]
    static_directory = "fiscalwrapper/static"
    images_path = os.path.join(data_dir, "bundles/{}".format(static_directory))
    position = "left" if is_larger_paper else "top"
    image_logo = "{}/logo_{}.png".format(images_path,  position)

    if not os.path.exists(image_logo):
        images_path = os.path.join(data_dir, "../../src/{}".format(static_directory.replace("/static", "/src/static")))
        image_logo = "{}/logo_{}.png".format(images_path, position)
        if not os.path.exists(image_logo):
            return None

    return image_logo


def _int_low_high(inp_number, out_bytes):
    """ Generate multiple bytes for a number: In lower and higher parts, or more parts as needed.

    :param inp_number: Input number
    :param out_bytes: The number of bytes to output (1 - 4).
    """
    max_input = (256 << (out_bytes * 8) - 1)
    if not 1 <= out_bytes <= 4:
        raise ValueError("Can only output 1-4 byes")
    if not 0 <= inp_number <= max_input:
        raise ValueError("Number too large. Can only output up to {0} in {1} byes".format(max_input, out_bytes))
    outp = b''
    for _ in range(0, out_bytes):
        outp += six.int2byte(inp_number % 256)
        inp_number //= 256
    return outp


def _image_send_graphics_data(m, fn, data):
    """
    Wrapper for GS ( L, to calculate and send correct data length.

    :param m: Modifier//variant for function. Usually '0'
    :param fn: Function number to use, as byte
    :param data: Data to send
    """
    header = _int_low_high(len(data) + 2, 2)
    return GS + b'(L' + header + m + fn + data


if __name__ == '__main__':
    pass



from helper import ExtendedEnum


class MwSysErrors(ExtendedEnum):

    SE_UNKOWNUSR = "5000"                               # unknown user id
    SE_INVALIDPASSWD = "5001"                           # invalid password
    SE_DAYNOTCLOSED = "5002"                            # business day is not closed
    SE_DAYNOTOPENED = "5003"                            # business day is not opened
    SE_OPERLOGGEDIN = "5004"                            # operator is still logged in
    SE_OPERNOTLOGGEDIN = "5005"                         # operator is not logged in
    SE_ORDEROPENED = "5006"                             # there are a sale order opened
    SE_OPEROPENED = "5007"                              # there are operators opened in the account database
    SE_ORDER_IS_ALREADY_OPENED = "8"                    # create order is already in progress



from xml.etree import cElementTree as Etree

from model.log import LogEntry
from _EventProcessor import EventProcessor


def _get_log_entry_from_pos_event(data, event_type):
    event = Etree.fromstring(data)
    event_data = event[0] if "Event" in event.tag else event

    return LogEntry(
        pos_id=event_data.get("posId", event_data.get("posid")) or -1,
        order_id=event_data.get("orderId"),
        business_period=event_data.get("businessPeriod", event_data.get("period")),
        event_type=event_type,
        session_id=event_data.get("sessionId", event_data.get("sessionid")),
        event_data=Etree.tostring(event_data)
    )


class PosEventProcessor(EventProcessor):
    def parse_data(self, data, event_type):
        # type: (str, str) -> LogEntry

        return _get_log_entry_from_pos_event(data, event_type)



# This file is dual licensed under the terms of the Apache License, Version
# 2.0, and the BSD License. See the LICENSE file in the root of this repository
# for complete details.
"""
Hazardous Materials

This is a "Hazardous Materials" module. You should ONLY use it if you're
100% absolutely sure that you know what you're doing because this module
is full of land mines, dragons, and dinosaurs with laser guns.
"""
from __future__ import absolute_import, division, print_function



from ._CustomStorage import CustomStorage


class Comment(CustomStorage):
    """ class Comment(object)
    Represents an item's comment
    """

    comment_id = ''
    text = ''

    def __init__(self, **kargs):
        CustomStorage.__init__(self)
        for key, val in kargs.iteritems():
            setattr(self, key, val)

    def __str__(self):
        return "Comment id={} text={}".format(self.comment_id, self.text)



from production.box import OrderSplitterBox
from production.treebuilder import BoxConfiguration

from ._BoxBuilder import BoxBuilder


class OrderSplitterBoxBuilder(BoxBuilder):
    def __init__(
        self,
        order_changer,
        publish_scheduler,
        loggers
    ):
        super(OrderSplitterBoxBuilder, self).__init__(loggers)
        self.order_changer = order_changer
        self.publish_scheduler = publish_scheduler
        self.split_type = None
        self.split_value = None

    def build(self, config):
        # type: (BoxConfiguration) -> OrderSplitterBox

        if u"SplitType" in config.extra_config:
            self.split_type = config.extra_config[u"SplitType"]
        if u"SplitValue" in config.extra_config:
            self.split_value = config.extra_config[u"SplitValue"]

        return OrderSplitterBox(
            config.name,
            config.sons,
            self.order_changer,
            self.publish_scheduler,
            self.split_type,
            self.split_value,
            self.get_logger(config)
        )



import datetime
import sys
from typing import Any, Mapping, Optional, Pattern, TypeVar


__all__ = ["parse_date", "ParseError", "UTC", "FixedOffset"]


if sys.version_info >= (3, 0, 0):
    basestring = str

if sys.version_info >= (3, 2):
    UTC: datetime.timezone = ...

    def FixedOffset(
        offset_hours: float, offset_minutes: float, name: str
    ) -> datetime.timezone:
        ...


else:
    ZERO: datetime.timedelta = ...

    class Utc(datetime.tzinfo):
        ...

    UTC: Utc = ...

    class FixedOffset(datetime.tzinfo):
        def __init__(
            self, offset_hours: float, offset_minutes: float, name: str
        ) -> None:
            ...


ISO8601_REGEX: Pattern[basestring] = ...


class ParseError(ValueError):

    ...


_T = TypeVar("_T")


def to_int(
    d: Mapping[_T, Any],
    key: _T,
    default_to_zero: bool = ...,
    default: Any = ...,
    required: bool = ...,
) -> int:
    ...


def parse_timezone(
    matches: Mapping[basestring, basestring],
    default_timezone: Optional[datetime.tzinfo] = ...,
) -> datetime.tzinfo:
    ...


def parse_date(
    datestring: basestring, default_timezone: Optional[datetime.tzinfo] = ...
) -> datetime.datetime:
    ...
